/* Terraform - (C) 1997-1999 Robert Gasch (Robert_Gasch@peoplesoft.com)
 *  - www.peoplesoft.com/peoplepages/g/robert_gasch/terraform/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */


#include <string.h>
#include "TFWindowMain.h"
#include "FileIO.h"
#include "HeightField.h"
#include "HeightFieldGenRandom.h"
#include "GlobalTrace.h"
#include "GlobalSanityCheck.h"
#include "MenuDefs.h"
#include "TFDialogGenSSynth.h"
#include "TFDialogGenSubdiv.h"
#include "TFGui.h"					// circular dependency
#include "TFWindowHandler.h"

#ifdef HAVE_IMLIB
#include "GuiDialogAboutDissolve.h"
#endif

extern int readhf (HeightField **HF, char *fname, bool haveGUI);


/*
 *  constructor: build the window with an itemfactory, buffered drawing area 
 * 	and colormaps. Store the generated menu in a hash table so we have 
 *	a handy reference later we can use for set_sensitvity calls
 */
TFWindowMain::TFWindowMain(FlexArray *wList) : TFWindowBase (wList),
		d_accelGroup (),
		d_vBoxList (FALSE, 5),
		d_vBoxMenu (FALSE, 0),
		d_vBoxMain (FALSE, 0),
		d_win (GTK_WINDOW_TOPLEVEL)
{
	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "+++ TFWindowMain\n");

	d_cList = NULL;
	d_genSSynthDialog = d_genSubdivDialog = NULL;
	d_fileSelection = NULL;

	buildWindow ();
	buildCListMenu ();
}


/*
 *  destructor: clean up 
 */
TFWindowMain::~TFWindowMain()
{
	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "--- TFWindowMain\n");
}


/*
 *  quit: clean up and exit
 */
void TFWindowMain::quit ()
{
	gtk_exit (0);
}


/*
 *  buildWindow: assemble the widgets and add the menu 
 */
void TFWindowMain::buildWindow ()
{
	char			*s;
	GtkStyle		*style; 
	Gtk_Style		Style; 

	d_win.add (d_vBoxMain);

	d_itemFactory = new Gtk_ItemFactory_MenuBar("<Main>", d_accelGroup);
	style = Style.gtkobj();
	gdk_font_unref (style->font);
	style->font = gdk_font_load ("-*-Helvetica-Medium-R-Normal--*-100-*-*-*-*-*-*");

    	//d_menuBarHandle = static_cast<Gtk_ObjectHandle<Gtk_MenuBar>&> (d_itemFactory->get_menubar_widget(""));
	{
	// uglier than above but without compiler warning 
	Gtk_ObjectHandle<Gtk_MenuBar> a(d_itemFactory->get_menubar_widget(""));
    	d_menuBarHandle = a;
	}

	SanityCheck::bailout ((!d_menuBarHandle.get_object()), "d_menuBarHandle.get_object failed", "TFWindowMain::TFWindowMain");
	d_menuBarHandle->push_style (Style);

	// create the menu items
	s = strdup (MENU_FILE_BRANCH);
	d_itemFactory->create_item (s, TFWindow::buildAccel (MENU_FILE_BRANCH, ACCEL_TYPE_ALT), "<Branch>", 0);
	s = strdup (MENU_FILE_NEW_BRANCH);
	d_itemFactory->create_item (s, 0, "<Branch>", 0);
	s = strdup (MENU_FILE_NEW_SYNTHESIS);
	d_itemFactory->create_item (s, TFWindow::buildAccel (MENU_FILE_NEW_SYNTHESIS, ACCEL_TYPE_CNTRL), "", 
		ItemFactoryConnector<TFWindowMain, string>(this,&TFWindowMain::fileMenuCallback, s));
	s = strdup (MENU_FILE_NEW_SUBDIVIDE);
	d_itemFactory->create_item (s, TFWindow::buildAccel (MENU_FILE_NEW_SUBDIVIDE, ACCEL_TYPE_CNTRL), "", 
		ItemFactoryConnector<TFWindowMain, string>(this,&TFWindowMain::fileMenuCallback, s));
	d_itemFactory->create_item ("/File/New/", 0, "<Separator>", 0);
	s = strdup (MENU_FILE_NEW_RANDOM);
	d_itemFactory->create_item (s, TFWindow::buildAccel (MENU_FILE_NEW_RANDOM, ACCEL_TYPE_CNTRL), "", 
		ItemFactoryConnector<TFWindowMain, string>(this,&TFWindowMain::fileMenuCallback, s));

	s = strdup (MENU_FILE_OPEN);
	d_itemFactory->create_item (s, TFWindow::buildAccel (MENU_FILE_OPEN, ACCEL_TYPE_CNTRL), "", 
	    ItemFactoryConnector<TFWindowMain, string>(this,&TFWindowMain::fileMenuCallback, s));

	d_itemFactory->create_item ("/File/", 0, "<Separator>", 0);
	s = strdup (MENU_FILE_OPTIONS);
	d_itemFactory->create_item (s, TFWindow::buildAccel (MENU_FILE_OPTIONS, ACCEL_TYPE_CNTRL), "", 
	    ItemFactoryConnector<TFWindowMain, string>(this,&TFWindowMain::fileMenuCallback, s));
	s = strdup (MENU_FILE_ABOUT);
	d_itemFactory->create_item (s, TFWindow::buildAccel (MENU_FILE_ABOUT, ACCEL_TYPE_CNTRL), "", 
	    ItemFactoryConnector<TFWindowMain, string>(this,&TFWindowMain::fileMenuCallback, s));

	d_itemFactory->create_item ("/File/", 0, "<Separator>" ,0);
	s = strdup (MENU_FILE_EXIT);
	d_itemFactory->create_item (s, TFWindow::buildAccel (MENU_FILE_EXIT, ACCEL_TYPE_CNTRL), "",
	    ItemFactoryConnector<TFWindowMain, string>(this,&TFWindowMain::fileMenuCallback, s));

	// add accel group + menu bar to menu vbox
	d_vBoxMain.pack_start (d_vBoxMenu, FALSE, FALSE, 0);
	d_win.add_accel_group (d_accelGroup);
	d_vBoxMenu.pack_start (*d_menuBarHandle, FALSE, FALSE, 0);

	// add List VBox holding CList
	d_vBoxList.set_border_width (5);
	d_vBoxMain.pack_start (d_vBoxList, TRUE, TRUE, 0);
	d_cList = new Gtk_CList (1);
	d_cList->set_column_title (0, "Terrain Object List");
	d_cList->column_titles_show ();
	d_cList->set_shadow_type (GTK_SHADOW_OUT);
	d_vBoxList.pack_start (*d_cList, TRUE, TRUE, 0);

	// catch destroy event
	connect_to_method (d_win.destroy, this, &TFWindowMain::quit);

	d_win.show ();
	d_win.set_usize (200, 300);
	d_win.set_policy (FALSE, FALSE, FALSE);
	d_cList->show ();
	d_vBoxList.show ();
	d_vBoxMenu.show ();
	d_vBoxMain.show ();
	d_menuBarHandle->show();

	Gtk_Main *gtkMain = Gtk_Main::instance();
	while (gtkMain->events_pending())
		gtkMain->iteration ();
}


/*
 *  buildCListMenu: build the menu used by the CList.
 */
void TFWindowMain::buildCListMenu ()
{
	char			*s;
	GtkStyle		*style; 
	Gtk_Style		Style; 

	d_itemFactoryCList = new Gtk_ItemFactory_Menu ("<CList>", d_accelGroupCList);
	style = Style.gtkobj();
	gdk_font_unref (style->font);
	style->font = gdk_font_load ("-*-Helvetica-Medium-R-Normal--*-100-*-*-*-*-*-*");

    //d_menuBarHandle = static_cast<Gtk_ObjectHandle<Gtk_MenuBar>&> (d_itemFactory->get_menubar_widget(""));
	{
	// uglier than above but without compiler warning 
	Gtk_ObjectHandle<Gtk_Menu> a(d_itemFactoryCList->get_menu_widget(""));
    d_menuCListHandle = a;
	}

	SanityCheck::bailout ((!d_menuCListHandle.get_object()), "d_menuCListHandle.get_object failed", "TFWindowMain::TFWindowMain");
	d_menuCListHandle->push_style (Style);

	// create the menu items
	//s = strdup (MENU_CLIST_BRANCH);
	//d_itemFactoryCList->create_item (s, NULL, "<Branch>", 0);
	s = strdup (MENU_CLIST_HIDE);
	d_itemFactoryCList->create_item (s, NULL, "", 
		ItemFactoryConnector<TFWindowMain, string>(this,&TFWindowMain::fileMenuCallback, s));
	s = strdup (MENU_CLIST_SHOW);
	d_itemFactoryCList->create_item (s, NULL, "", 
		ItemFactoryConnector<TFWindowMain, string>(this,&TFWindowMain::fileMenuCallback, s));
}


/*
 *  fileMenuCallback: process a menu choice in the file menu
 */
void TFWindowMain::fileMenuCallback (string m)
{
	int		rc=0;
	char		*menuitem = const_cast<char*>(m.c_str()),
			*aboutText[] = { "brought to you by", 
					"Robert_Gasch@peoplesoft.com", };
	HeightField	*HF = NULL;

	if (!strcmp (menuitem, MENU_FILE_NEW_SYNTHESIS))
		{
		HF = new HeightField ();
		if (!d_genSSynthDialog)
			d_genSSynthDialog = new TFDialogGenSSynth (HF);
		else
			{
			TFDialogGenSSynth 	*TFDGen;

			TFDGen = dynamic_cast<TFDialogGenSSynth*>(d_genSSynthDialog);
			TFDGen->refresh ();
			TFDGen->set_title (HeightField::getNewName());
			}
		
		d_genSSynthDialog->show ();
		}
	else
	if (!strcmp (menuitem, MENU_FILE_NEW_SUBDIVIDE))
		{
		HF = new HeightField ();
		if (!d_genSubdivDialog)
			d_genSubdivDialog = new TFDialogGenSubdiv (HF);
		else
			{
			TFDialogGenSubdiv *TFDGen;

			TFDGen = dynamic_cast<TFDialogGenSubdiv*>(d_genSubdivDialog);
			TFDGen->refresh ();
			TFDGen->set_title (HeightField::getNewName());
			}
		
		d_genSubdivDialog->show ();
		}
	else
	if (!strcmp (menuitem, MENU_FILE_NEW_RANDOM))
		{
		HF = new HeightField ();
		HeightFieldGenRandom::generate (HF, 400);
		TFGui::addHeightField (HF);
		}
	else
	if (!strcmp (menuitem, MENU_FILE_OPEN))
		{
		fileSelection ();
		}
	else
	if (!strcmp (menuitem, MENU_FILE_ABOUT))
		{
		if (!this->s_aboutDialog)
			{
#ifdef HAVE_IMLIB
			char	buf[256];
			sprintf (buf, "%s/terraform-logo.jpg", TF_DATADIR);
			FileIO	*fio = new FileIO (buf);
			if (fio->exists())
				this->s_aboutDialog = new GuiDialogAboutDissolve ("About Terraform", buf, aboutText, 2);
			else
				fprintf (stderr, "Error opening %s\n", buf);
			delete fio;
#else
			this->s_aboutDialog = new GuiDialogAbout ("About Terraform", NULL, aboutText, 2);
#endif
			if (this->s_aboutDialog)
				this->s_aboutDialog->buildDialogWindow ();
			}

		if (this->s_aboutDialog)
			this->s_aboutDialog->show ();
		}
	else
	if (!strcmp (menuitem, MENU_FILE_OPTIONS))
		{
		if (!this->s_optionsDialog)
			this->s_optionsDialog = new TFDialogOptions (TFWindowBase::s_winList);
		else
			this->s_optionsDialog->show ();
		}
	else
	if (!strcmp (menuitem, MENU_FILE_EXIT))
		{
		if (!TFWindowHandler::allWindowsSaved())
			{
			if (!this->s_quitDialog)
				this->s_quitDialog = new GuiDialogReallyQuit ("", "");
			else
				this->s_quitDialogConnection.disconnect ();

			this->s_quitDialog->set_title ("Really Quit Terraform?");
			this->s_quitDialog->setLabel ("Unsaved Changes! Exit anyway?");
			this->s_quitDialogConnection = connect_to_method (
				this->s_quitDialog->getYesButton()->clicked, 
				this, &TFWindowMain::quit);
			this->s_quitDialog->show ();
			}
		else
			quit ();
		}
	else
		SanityCheck::bailout (TRUE, "uncaught <file> menuitem", "TFWindowHandler::fileMenuCallback");

	if (rc)
		g_print ("Warning: last command returned an Error (%d)\n", rc);
	fflush (stdout);
}


/*
 *  fileSelectionOK: user pressed OK in file selection dialog
 */
void TFWindowMain::fileSelectionOK ()
{
	gchar 	*fname;
	int	rc;
	HeightField	*HF=NULL;
	TFWindowHandler	*tfWin=NULL;

	d_fileSelection->hide ();

	fname = const_cast<char*>(d_fileSelection->get_filename().c_str());

	rc = readhf (&HF, fname, TRUE);
	tfWin = new TFWindowHandler (TFWindowBase::s_winList);
	tfWin->setHeightField (HF);

	if (rc)
		cout << "Warning: last command returned Error " << rc << "\n";
	else
		TFWindowBase::s_winList->append (tfWin);
}


/*
 *  fileSelectionCancel: user pressed Cancel in file selection dialog
 */
void TFWindowMain::fileSelectionCancel ()
{
	d_fileSelection->set_filename ("");
	d_fileSelection->hide ();
}


/*
 *  fileSelection: open a file selection dialog
 */
void TFWindowMain::fileSelection ()
{
	if (!d_fileSelection)
		{
		d_fileSelection = new Gtk_FileSelection ("");
		d_fileSelection->set_position (GTK_WIN_POS_MOUSE);

		connect_to_method (d_fileSelection->get_ok_button()->clicked, 
					this, &TFWindowMain::fileSelectionOK);
		connect_to_method (d_fileSelection->get_cancel_button()->clicked,
					this, &TFWindowMain::fileSelectionCancel);
		}

	d_fileSelection->set_title ("Load Height Field");
	d_fileSelection->show ();
}
